'''
'''
import threading
import magpy.project as mp
import magpy.general as mg
from magpy.csv_files.backlog_file import BacklogFile
from magpy.csv_files.event_file import EventFile
from magpy.csv_files.sprint_file import SprintFile


class Hubs:
    '''
    Represents a Magshimim list of hubs, and all of their projects
    '''

    def __init__(self, gl, hubs):
        self.events = []
        self.backlog = []
        self.sprint_items = []

        print('Loading projects:')
        print('=================')
        for hub in hubs:
            try:
                group = gl.gitlab.groups.get(hub.group_id)

            except BaseException:
                print(f'Gitlab hub group not found: id = {hub.group_id}')
                exit(1)

            # Projects
            hub_projects = group.projects.list(include_subgroups=True,
                                               simple=True,
                                               all=True)

            threads = []
            events = [[] for i in range(len(hub_projects))]
            backlog = [[] for i in range(len(hub_projects))]
            sprints = [[] for i in range(len(hub_projects))]

            # Each project data extraction id done ina seperate thread
            for i, p in enumerate(hub_projects):
                gl_project = gl.gitlab.projects.get(p.id)
                thread_args = [gl, hub.name, hub.start_date, hub.end_date,
                               gl_project, i, events, backlog, sprints]
                thread = threading.Thread(target=self.get_project_data,
                                          args=thread_args)
                thread.start()
                threads.append(thread)

            # wait for all threads to end
            for thread in threads:
                thread.join()

            # collect thread results
            self.events += [e for p_events in events for e in p_events]
            self.backlog += [i for p_issues in backlog for i in p_issues]
            self.sprint_items += [
                i for p_sprints in sprints for s in p_sprints for i in s.items]

    def write_csv_files(self):
        '''
        Generate hubs csv data files
        '''
        print()
        print('Writing CSV files:')
        print('==================')
        BacklogFile(self.backlog).write()
        EventFile(self.events).write()
        SprintFile(self.sprint_items).write()

    def get_project_data(self, gl, hub_name, start_date, end_date,
                         gl_project, i, events, backlog, sprints):
        '''
        Extracts project data from Gitlab
        '''
        project = mp.Project(gl, hub_name, start_date, end_date, gl_project)

        # Extracted data is kept in a list to avoid thread colissions
        events[i] = project.events
        backlog[i] = project.backlog
        sprints[i] = project.sprints



import magpy.csv_files.base as mc


class SprintFile(mc.CsvFile):
    '''
    '''
    __NAME = 'sprints.csv'
    __HEADER = ('מוקד',
                'פרויקט',
                'ספרינט',
                'סוג',
                'מספר',
                'משימה',
                'חניך',
                'סטטוס',
                'שעות')

    def __init__(self, sprints):
        super().__init__(file_name=SprintFile.__NAME,
                         header=SprintFile.__HEADER,
                         row=lambda sprint_item: (sprint_item.hub_name,
                                                  sprint_item.project_name,
                                                  sprint_item.sprint_name,
                                                  sprint_item.type,
                                                  sprint_item.number,
                                                  sprint_item.title,
                                                  sprint_item.assignee,
                                                  sprint_item.status,
                                                  sprint_item.work),
                         data_list=sprints)



import datetime
import operator
import time
import threading
import magpy.general as mg
from magpy.events.project_event import ProjectEvent
from magpy.events.label_event import LabelEvent
from magpy.sprint_items.sprint_issue import SprintIssue
from magpy.sprint_items.sprint_balance import SprintBalance
from magpy.sprint_items.sprint_progress import SprintProgress
from magpy.sprint import Sprint
from magpy.backlog_issue import BacklogIssue


class Project:
    '''
    Represents a Magshimim project, with all its data
    '''

    def __init__(self, gl, hub_name, start_date, end_date, gl_project):
        start = time.time()

        # Project
        self.project = gl_project
        self.name = gl_project.name
        self.start_date = start_date
        self.end_date = end_date
        issues = self.project.issues.list(all=True, simple=True)
        events = self.project.events.list(all=True)

        # Group
        self.group = gl.gitlab.groups.get(self.project.namespace.get('id'))

        # Hub name
        self.hub_name = hub_name

        # Sprints
        self.sprints = self.__sprints(issues)

        # Events
        self.events = self.__project_events(events)
        self.events += self.__label_events(issues)

        # Backlog
        self.backlog = self.__backlog(issues)

        elapsed = round(time.time() - start, 1)
        print(f'{self.hub_name}: {self.name} loaded in {elapsed}s')

    def __project_events(self, gl_events):
        '''
        Returns a list of project events
        '''
        events = []
        sprint_start_dates = [s.start_date for s in self.sprints]
        for gl_event in gl_events:
            user_name = gl_event.author.get(mg.EVENT_NAME)
            event = ProjectEvent(hub_name=self.hub_name,
                                 project_name=self.name,
                                 user_name=user_name,
                                 sprint_start_dates=sprint_start_dates,
                                 gl_event=gl_event)
            events.append(event)
        return events

    def __label_events(self, gl_issues):
        '''
        Returns a list of label events, for each related issue
        '''
        threads = []
        issue_events = [[] for i in range(len(gl_issues))]
        sprint_start_dates = [s.start_date for s in self.sprints]
        for i, gl_issue in enumerate(gl_issues):
            thread = threading.Thread(target=self.get_label_events,
                                      args=[gl_issue, self.hub_name,
                                            self.name,
                                            sprint_start_dates,
                                            i, issue_events])
            thread.start()
            threads.append(thread)

        # Wait for all threads to complete
        for thread in threads:
            thread.join()

        # Combine all events to a sinle list
        events = [event for events in issue_events for event in events]

        return events

    def __backlog(self, issues):
        '''
        Returns the project backlog
        '''
        backlog = []
        for issue in issues:
            issue_is_open = issue.state == mg.ISSUE_STATE_OPENED
            not_in_milestone = issue.milestone is None
            not_assigned = issue.assignee is None
            # only open issues, unassigned and not associated with a milestone
            # are considered as a backlog issue
            if issue_is_open and not_in_milestone and not_assigned:
                backlog.append(BacklogIssue(self, issue))

        return backlog

    def __sprints(self, issues):
        '''
        Returns a list of sprints and their associated issues.
        If no valid sprint (milestones) exists, consider the entire
        project as sprint 0.
        '''
        # Start date
        sprints = []
        for milestone in self.project.milestones.list():
            if milestone.start_date is not None and milestone.due_date is not None:
                # Milestones with start and due dates can be made into sprints
                start_date = mg.str_to_date(milestone.start_date)
                end_date = mg.str_to_date(milestone.due_date)
                sprint = Sprint(milestone, start_date, end_date)
                sprints.append(sprint)

        if len(sprints) > 0:
            sprints.sort(key=operator.attrgetter('start_date'))
        else:
            # No valid milestones => entire project is sprint 0
            sprints.append(Sprint(None, self.start_date, self.end_date))

        # Collect data for all sprint issues
        for n, sprint in enumerate(sprints):
            sprint.number = n
            allocated = 0

            # Sprint issues
            for gl_issue in issues:
                if gl_issue.milestone is None:
                    continue

                milestone_id = gl_issue.milestone.get(mg.MILESTONE_ID)
                if milestone_id != sprint.milestone_id:
                    continue

                issue = SprintIssue(self, sprint, gl_issue)
                sprint.items.append(issue)

                allocated += issue.work

            # Sprint balance: unallocated time vs budget
            if sprint.budget > 0:
                balance = sprint.budget - allocated
                sprint_balance = SprintBalance(self, sprint, balance)
                sprint.items.append(sprint_balance)

            # Sprint progress: estimated percentage of sprint at current time
            progress = SprintProgress(self, sprint)
            sprint.items.append(progress)

        return sprints

    def get_label_events(self, gl_issue, hub_name, project_name,
                         sprint_start_dates, i, events):
        '''
        Get all the label changes of an issue. Specifically, look
        for change event pairs, happening at the same time
        '''
        label_events = gl_issue.resourcelabelevents.list()

        remove_label_event = None
        for label_event in label_events:
            if label_event.label is None:
                continue

            if label_event.action == mg.LABEL_EVENT_REMOVE:
                remove_label_event = label_event
            else:
                if remove_label_event is not None:
                    same_time_events = label_event.created_at == remove_label_event.created_at
                    if same_time_events:
                        # Change issue status event (e.g., To Do => In
                        # Progress)
                        user_name = label_event.user.get(mg.LABEL_EVENT_NAME)
                        event = LabelEvent(hub_name=hub_name,
                                           project_name=project_name,
                                           user_name=user_name,
                                           sprint_start_dates=sprint_start_dates,
                                           issue_number=gl_issue.iid,
                                           issue_title=gl_issue.title,
                                           from_label_event=remove_label_event,
                                           to_label_event=label_event)
                        events[i].append(event)

                elif label_event.label.get(mg.LABEL_EVENT_NAME) in \
                        (mg.LABEL_TO_DO, mg.LABEL_IN_PROGRESS, mg.LABEL_DONE):
                    # Set issue initial status (e.g., => To Do)
                    user_name = label_event.user.get(mg.LABEL_EVENT_NAME)
                    event = LabelEvent(hub_name=hub_name,
                                       project_name=project_name,
                                       user_name=user_name,
                                       sprint_start_dates=sprint_start_dates,
                                       issue_number=gl_issue.iid,
                                       issue_title=gl_issue.title,
                                       from_label_event=None,
                                       to_label_event=label_event)
                    events[i].append(event)



import datetime
import gitlab as gl
import magpy.general as mg


class Event:
    '''
    '''
    TARGET_TYPE_GITLAB = 'GitLab'
    TARGET_TYPE_REPOSITOTY = 'Repository'

    def __init__(self, hub_name, project_name, user_name,
                 sprint_start_dates, gl_event_date):
        self.hub_name = hub_name
        self.project_name = project_name
        self.student_name = self.__user_name(user_name)
        event_datetime = self.__datetime(gl_event_date)
        self.date = event_datetime.date()
        self.time = event_datetime.time()
        self.week = self.__week(self.date, sprint_start_dates[0])
        self.sprint_number = self.__sprint_number(
            self.date, sprint_start_dates)
        self.sprint_week = self.__sprint_week(
            self.sprint_number, self.date, sprint_start_dates)

    def __user_name(self, user_name):
        '''
        Returns the name of the user
        '''
        name = user_name.strip().title().split()
        student_name = name[0]
        # Last name initial (if exists)
        if len(name) > 1:
            student_name += f' {name[1][0]}.'

        return student_name

    def __datetime(self, event_datetime):
        '''
        Returns the event date and time in local time-zone
        '''

        return mg.str_to_datetime(event_datetime)

    def __sprint_number(self, event_date, project_sprint_start_dates):
        '''
        Returns the event sprint number
        '''
        sprint_number = -1
        for sprint_start in project_sprint_start_dates:
            if event_date < sprint_start:
                break
            sprint_number += 1

        if sprint_number == -1:
            sprint_number = 0

        return sprint_number

    def __week(self, event_date, start_date):
        '''
        Returns the week number in the year (1 = week of first meeting)
        '''

        return (event_date - start_date).days // 7 + 1

    def __sprint_week(self, sprint_num, event_date, sprint_start_dates):
        '''
        Represent sprint-week as a decimal namber, where:
           sprint number is the integer part and
           week number is the fraction
        Example: 2.3 represents week 3 of sprint 2
        '''
        sprint_week = 0
        for sprint_start in sprint_start_dates:
            if event_date >= sprint_start:
                days_in_sprint = (event_date - sprint_start).days
                sprint_week = days_in_sprint // 7 + 1

        return sprint_num + sprint_week / 10



import csv
import os


class CsvFile:
    '''
    '''

    def __init__(self, file_name, header, row, data_list):
        self.file_name = file_name
        self.data_rows = [header]
        for data_item in data_list:
            self.data_rows.append(row(data_item))

    def write(self):
        print(f'Writing file {self.file_name}: {len(self.data_rows)} rows')
        with open(self.file_name, 'w', newline='') as csv_file:
            writer = csv.writer(csv_file, delimiter=',', quotechar='"')
            writer.writerows(self.data_rows)



import magpy.general as mg
import magpy.sprint_items.base as ms


class SprintIssue(ms.SprintItem):
    '''
    '''

    def __init__(self, project, sprint, gitlab_issue):
        super().__init__(project, sprint)

        self.title = gitlab_issue.title
        self.number = gitlab_issue.iid
        self.assignee = ''
        if gitlab_issue.assignee is not None:
            self.assignee = gitlab_issue.assignee.get(mg.ISSUE_ASSIGNEE_NAME)
        self.assignee = self.assignee.strip().title()
        self.status, self.type = self.__status(gitlab_issue)
        self.work = self.__work(gitlab_issue)

    def __status(self, gitlab_issue):
        '''
        Returns the status of an issue
        '''
        issue_is_closed = gitlab_issue.state == mg.ISSUE_STATE_CLOSED
        issue_is_done = mg.LABEL_DONE in gitlab_issue.labels
        issue_is_in_progress = mg.LABEL_IN_PROGRESS in gitlab_issue.labels
        if issue_is_closed or issue_is_done:
            return ('v', mg.LABEL_DONE)
        elif issue_is_in_progress:
            return ('*', mg.LABEL_IN_PROGRESS)
        else:
            return ('', mg.LABEL_TO_DO)

    def __work(self, gitlab_issue):
        '''
        Returns the estimated (or spent) work time of an issue, in hours
        '''
        estimate = gitlab_issue.time_stats().get(mg.ISSUE_TIME_ESTIMATE) / 3600
        spent = gitlab_issue.time_stats().get(mg.ISSUE_TIME_SPENT) / 3600
        return spent if spent > 0 else estimate



import datetime
import magpy.sprint_items.base as ms


class SprintProgress(ms.SprintItem):
    '''
    '''

    __PROGRESS_NAME = 'היום'
    __PROGRESS_TITLE = 'התקדמות צפויה'

    def __init__(self, project, sprint):
        super().__init__(project, sprint)

        self.type = SprintProgress.__PROGRESS_NAME
        self.title = SprintProgress.__PROGRESS_TITLE
        today = datetime.date.today()
        days_from_today = today - sprint.start_date
        days_of_sprint = sprint.end_date - sprint.start_date
        progress = min(round(days_from_today / days_of_sprint, 2), 1.0)
        self.work = progress if sprint.start_date < today else 0.0



import magpy.general as mg
from magpy.events.base import Event


class ProjectEvent(Event):
    '''
    GitLab project event
    '''
    __NEW_PROJECT = 'new project'

    def __init__(self,
                 hub_name,
                 project_name,
                 user_name,
                 sprint_start_dates,
                 gl_event):

        super().__init__(hub_name,
                         project_name,
                         user_name,
                         sprint_start_dates,
                         gl_event.created_at)

        self.action_name = gl_event.action_name.capitalize()
        self.target_type = self.__target_type(gl_event.target_type)
        self.action_details = self.__action_details(gl_event)

    def __target_type(self, target_type):
        '''
        Event can be either a Gitlab event or a Repository one
        '''
        if target_type is None:
            return Event.TARGET_TYPE_REPOSITOTY

        return Event.TARGET_TYPE_GITLAB

    def __action_details(self, gl_event):
        '''
        Returns a string with the event action details
        '''
        action_text = gl_event.action_name.capitalize()
        if gl_event.target_type is not None:
            if gl_event.action_name == mg.EVENT_ACTION_COMMENTED_ON:
                action_subject = gl_event.note.get(mg.NOTEABLE_TYPE)
                action_text += f' {action_subject.lower()}'
            else:
                action_text += f' {gl_event.target_type.lower()}'
            if gl_event.target_title is not None:
                action_text += f': {gl_event.target_title}'

        # Add issue number
        issue_number = None
        if gl_event.action_name in (mg.EVENT_ACTION_OPENED,
                                    mg.EVENT_ACTION_CLOSED):
            issue_number = gl_event.target_iid
        elif gl_event.action_name == mg.EVENT_ACTION_COMMENTED_ON:
            issue_number = gl_event.note.get(mg.NOTEABLE_IID)

        if issue_number is not None:
            action_text += f' (#{issue_number})'

        # Pushed to repository
        if gl_event.action_name == mg.EVENT_ACTION_PUSHED_TO:
            action_text += f' {gl_event.push_data.get(mg.PUSH_REF_TYPE)}'
            action_text += f' {gl_event.push_data.get(mg.PUSH_REF)}'
            commit_title = gl_event.push_data.get(mg.PUSH_COMMIT_TITLE)
            if commit_title is not None:
                action_text += f': {commit_title}'

        # Pushed new branch
        if gl_event.action_name == mg.EVENT_ACTION_PUSHED_NEW:
            action_text += ' ' + gl_event.push_data.get(mg.PUSH_REF_TYPE)
            branch_name = gl_event.push_data.get(mg.PUSH_REF)
            if branch_name is not None:
                action_text += f' {branch_name}'

        # Created new branch/repository
        if gl_event.action_name == mg.EVENT_ACTION_CREATED:
            action_text += f' {ProjectEvent.__NEW_PROJECT}'

        return action_text



class SprintItem:
    '''
    '''

    def __init__(self, project, sprint):
        self.hub_name = project.hub_name
        self.project_name = project.name
        self.sprint_name = f'ספרינט {str(sprint.number)}'
        self.number = 0
        self.assignee = ''
        self.status = ''



import magpy.csv_files.base as mc


class EventFile(mc.CsvFile):
    '''
    '''

    __NAME = 'events.csv'
    __HEADER = ('מוקד',
                'פרויקט',
                'תאריך',
                'זמן',
                'חניך',
                'ספרינט',
                'שבוע',
                'ספרינט-שבוע',
                'סוג פעולה',
                'נושא',
                'פעולה')

    def __init__(self, events):
        super().__init__(file_name=EventFile.__NAME,
                         header=EventFile.__HEADER,
                         row=lambda event: (event.hub_name,
                                            event.project_name,
                                            event.date,
                                            event.time,
                                            event.student_name,
                                            event.sprint_number,
                                            event.week,
                                            event.sprint_week,
                                            event.action_name,
                                            event.target_type,
                                            event.action_details),
                         data_list=events)



import magpy.csv_files.base as mc


class BacklogFile(mc.CsvFile):
    '''
    '''
    __NAME = 'backlog.csv'
    __HEADER = ('מוקד',
                'פרויקט',
                'תאריך',
                'זמן',
                'גיל',
                'מספר',
                'משימה')

    def __init__(self, backlog):
        super().__init__(file_name=BacklogFile.__NAME,
                         header=BacklogFile.__HEADER,
                         row=lambda backlog_issue: (backlog_issue.hub_name,
                                                    backlog_issue.project_name,
                                                    backlog_issue.date,
                                                    backlog_issue.time,
                                                    backlog_issue.age,
                                                    backlog_issue.issue_number,
                                                    backlog_issue.issue_title),
                         data_list=backlog)



import datetime
import magpy.general as mg


class Sprint:
    '''
    A project iterration of ~3 weeks, with specific target and contents
    '''

    def __init__(self, gl_milestone, start_date, end_date):
        if gl_milestone is not None:
            # Milestone-based sprint
            self.name = gl_milestone.title
            self.milestone_id = gl_milestone.id
            self.budget = self.__budget(gl_milestone.description)
        else:
            # Default sprint
            self.name = ''
            self.milestone_id = 0
            self.budget = 0

        self.start_date = start_date
        self.end_date = end_date
        self.items = []
        self.number = -1

    def __budget(self, gl_milestone_desc):
        '''
        Extracts sprint budget from the milestone description, based
        on convention, for example: if thr first line includes ' = 4h',
        it means that the budget for the sprint is 4 hours.
        If convention is not followed, budget is set to 0 hours.
        '''
        try:
            desc_first_line = gl_milestone_desc.splitlines()[0]
            budget_total = desc_first_line.split(' = ')[1]
            budget = int(budget_total.split('h')[0])
        except BaseException:
            budget = 0

        return budget



import magpy.general as mg
from magpy.events.base import Event


class LabelEvent(Event):
    '''
    Events of label change on an issue (e.g., 'To Do' => 'In Progress')
    '''
    ACTION_MOVED = 'Moved'

    def __init__(self,
                 hub_name,
                 project_name,
                 user_name,
                 sprint_start_dates,
                 issue_number,
                 issue_title,
                 from_label_event,
                 to_label_event):

        super().__init__(hub_name,
                         project_name,
                         user_name,
                         sprint_start_dates,
                         to_label_event.created_at)

        self.target_type = Event.TARGET_TYPE_GITLAB
        self.action_name = LabelEvent.ACTION_MOVED
        if from_label_event is not None:
            from_label = from_label_event.label.get(mg.LABEL_EVENT_NAME)
        else:
            from_label = ''
        to_label = to_label_event.label.get(mg.LABEL_EVENT_NAME)
        self.action_details = f'{from_label} => {to_label}'
        self.action_details += f': {issue_title} (#{issue_number})'



import setuptools

# Increase version number
with open("version.txt", "r") as vf:
    current_version = vf.read()
    major, minor, patch = current_version.split('.')

with open("version.txt", "w") as vf:
    # Major Release
    # major = int(major) + 1
    # minor = 0
    # patch = 0

    # Minor Release
    # minor = int(minor) + 1
    # patch = 0

    # Patch Release
    patch = int(patch) + 1

    current_version = f'{major}.{minor}.{patch}'
    vf.write(current_version)

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="magpy-raz",
    version=current_version,
    author="Raz Nitzan",
    author_email="raz.nitzan@gmail.com",
    description="Extract project data from GitLab",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/Razinka/magpy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import magpy.general as mg


class Hub:
    '''
    '''

    def __init__(self, name, group_id, start_date, end_date):
        self.name = name
        self.group_id = group_id
        self.start_date = mg.str_to_date(start_date)
        self.end_date = mg.str_to_date(end_date)



import magpy.sprint_items as ms


class SprintBalance(ms.base.SprintItem):
    '''
    '''
    __BALANCE_TYPE = 'יתרה'
    __BALANCE_TITLE = 'זמן פנוי למשימות'

    def __init__(self, project, sprint, balance):
        super().__init__(project, sprint)

        self.type = SprintBalance.__BALANCE_TYPE
        self.title = SprintBalance.__BALANCE_TITLE
        self.work = balance



import datetime
import magpy.general as mg


class BacklogIssue:
    '''
    A GitLab issue with no association to a milestone or assignee
    '''
    __TWO_WEEKS = 'שבועיים'
    __ONE_MONTH = 'חודש'
    __MORE_THAN_A_MONTH = 'יותר מחודש'

    def __init__(self, project, gl_issue):
        self.hub_name = project.hub_name
        self.project_name = project.name
        self.issue_number = gl_issue.iid
        self.issue_title = gl_issue.title
        self.date = mg.str_to_datetime(gl_issue.created_at).date()
        self.time = mg.str_to_datetime(gl_issue.created_at).time()
        self.age = self.__age(self.date)

    def __age(self, creation_date):
        '''
        '''

        today = datetime.date.today()
        age_days = (today - creation_date).days
        if age_days <= 14:
            return BacklogIssue.__TWO_WEEKS
        elif age_days < 30:
            return BacklogIssue.__ONE_MONTH
        else:
            return BacklogIssue.__MORE_THAN_A_MONTH



import gitlab as gl
import datetime
import pytz
import tzlocal

DATE_FORMAT = '%Y-%m-%d'
UTC_TIME_FORMAT = 'T%H:%M:%S.%fZ'

ISSUE_STATE_CLOSED = 'closed'
ISSUE_STATE_OPENED = 'opened'

LABEL_TO_DO = 'To Do'
LABEL_IN_PROGRESS = 'In Progress'
LABEL_DONE = 'Done'

EVENT_NAME = 'name'

MILESTONE_ID = 'id'

EVENT_ACTION_COMMENTED_ON = 'commented on'
EVENT_ACTION_CREATED = 'created'
EVENT_ACTION_OPENED = 'opened'
EVENT_ACTION_CLOSED = 'closed'
EVENT_ACTION_PUSHED_NEW = 'pushed new'
EVENT_ACTION_PUSHED_TO = 'pushed to'

NOTEABLE_TYPE = 'noteable_type'
NOTEABLE_IID = 'noteable_iid'

PUSH_REF = 'ref'
PUSH_REF_TYPE = 'ref_type'
PUSH_COMMIT_TITLE = 'commit_title'

LABEL_EVENT_REMOVE = 'remove'
LABEL_EVENT_NAME = 'name'

ISSUE_ASSIGNEE_NAME = 'name'
ISSUE_TIME_ESTIMATE = 'time_estimate'
ISSUE_TIME_SPENT = 'total_time_spent'


class MagGitlab:
    '''
    Gitlab connection and common functions
    '''

    def __init__(self, token_file):
        self.gitlab = None
        try:
            with open(token_file, 'r') as token_file:
                token = token_file.read().replace('\n', '')
                self.gitlab = gl.Gitlab(
                    'https://gitlab.com', private_token=token)
        except FileNotFoundError:
            print(f'File {token_file} not found.')
            exit(1)


def str_to_date(gitlab_date):
    '''
    Convert Gitlab date string to date
    '''
    return datetime.datetime.strptime(gitlab_date, DATE_FORMAT).date()


def str_to_datetime(gitlab_datetime):
    '''
    Convert GitLab UTC date-time string to local date-time
    '''
    utc_datetime = datetime.datetime.strptime(gitlab_datetime,
                                              DATE_FORMAT + UTC_TIME_FORMAT)
    return tzlocal.get_localzone().fromutc(utc_datetime)
